package com.bergerkiller.bukkit.nolagg.sending;

import java.util.LinkedList;
import java.util.Queue;

import org.bukkit.block.BlockFace;
import org.bukkit.entity.Player;

import com.bergerkiller.bukkit.nolagg.BufferedChunk;
import com.bergerkiller.bukkit.nolagg.Util;

import net.minecraft.server.Chunk;
import net.minecraft.server.EntityPlayer;
import net.minecraft.server.World;

public class ChunkCompressQueue {
    public ChunkCompressQueue(final ChunkSendQueue owner) {
        this.owner = owner;
    }
    private final ChunkSendQueue owner;
    private final Queue<Chunk> toCompress = new LinkedList<Chunk>();
    private final Queue<ChunkSendCommand> toSend = new LinkedList<ChunkSendCommand>();
    public void update(World world, int x, int z, BlockFace direction) {
    	
    }
    public void clear() {
    	synchronized (this.toCompress) {
    		this.toCompress.clear();
    	}
    	synchronized (this.toSend) {
    		this.toSend.clear();
    	}
    }
    public boolean isAlive() {
    	if (this.owner.ep.netServerHandler.disconnected) return false;
    	return true;
    }
    public boolean canSend() {
    	synchronized (this.toSend) {
    		return !this.toSend.isEmpty();
    	}
    }
    public boolean finishedCompressing() {
    	synchronized (this.toCompress) {
    		return this.toCompress.isEmpty();
    	}
    }
    public void enqueue(Chunk chunk) {
    	if (chunk == null) return;
        synchronized (this.toCompress) {
            this.toCompress.offer(chunk);
        }
    }
    public void enqueue(ChunkSendCommand sendCommand) {
    	if (sendCommand == null) return;
    	if (!sendCommand.isValid()) return;
        synchronized (this.toSend) {
            this.toSend.offer(sendCommand);
        }
    }
    public boolean isNear(Chunk chunk) {
    	return this.isNear(chunk.x, chunk.z);
    }
    public boolean isNear(int x, int z) {
    	synchronized (this.owner) {
    		return this.owner.isNear(x, z, Util.view);
    	}
    }
    public EntityPlayer nativeOwner() {
    	return this.owner.ep;
    }
    public Player owner() {
    	return this.owner.ep.getPlayer();
    }
    public Chunk pollChunk() {
    	Chunk chunk;
        synchronized (this.toCompress) {
        	if (this.toCompress.isEmpty()) return null;
            chunk = this.toCompress.poll();
        }
        if (!this.isNear(chunk)) return this.pollChunk();
        if (chunk instanceof BufferedChunk) {
        	if (((BufferedChunk) chunk).isFree()) {
        		return this.pollChunk();
        	}
        }
        return chunk;
    }
    public ChunkSendCommand pollSendCommand() {
        synchronized (this.toSend) {
        	if (this.toSend.isEmpty()) return null;
        	ChunkSendCommand cmd = this.toSend.poll();
        	if (!this.isNear(cmd.chunk)) return this.pollSendCommand(); 
        	return cmd;
        }
    }
    public boolean sendNext() {
    	ChunkSendCommand next = this.pollSendCommand();
        if (next == null) return false;
        if (!this.isNear(next.chunk)) return this.sendNext(); 
        next.send(this.owner.ep);
        return true;
    }
}
